/**<pre>
 * Title: ActiveCampus
 * Copyright: Copyright (c) 2002
 * Company:   University of Oregon Computer Science Dept.
 *
 * @author Jason Prideaux, adapted from Richard Caudle
 * @version 1.0
 * </pre>
 */

package app;

import app.activeCampus.gui.*;
import app.activeCampus.util.*;
import app.util.*;

import cat.app.*;
import cat.context.*;
import cat.sig.*;
import cat.sig.event.*;
import cat.hist.*;
import cat.sensor.core.*;
import cat.sensor.util.*;
import cat.sensor.event.*;
import cat.sensor.soft.*;
import cat.util.*;

import java.util.*;
import java.awt.*;
import java.awt.event.*;
import java.util.GregorianCalendar;
import java.io.IOException;


/* ====================================================================== */
/** This class is for running the ActiveCampus CAT Application.
 *
 */
public class ActiveCampus extends ContextApp{

	/** The GUI. */
  	private CampusFrame gui;


	//keep a hashmap of agents and their last known position.
	private HashMap table1 = new HashMap();

	//keep mapping of ids to agents.
	private HashMap table2 = new HashMap();

	//keep agents that are friends.
	private Vector friends = new Vector();

	//Just some junk for dealing with friend requests
	private Dialog notification;
	private String name;
	private Vector agents;
	private Vector wanted_sensors;
	private Vector sharing_sensors;



	/* ===================================================================== */
  	/** Constructor starts up the app.  Takes in the name and port
  	 *  of the agent starting this CAT app.
  	 *
  	 *  @param  id   The id of the CAT user agent.
  	 *  @param  port   The port of the CAT user agent.
  	 *
  	 */
  	public ActiveCampus(String id, int port){

		super(id, port);

		init();

    	//Set up the UI for the gps display
		gui = new CampusFrame(id,  this);

		//show the UI
		gui.setVisible(true);

  	} //Constructor




    /* ===================================================================== */
    /** Here is where we create any services/soft sensors that this application
     *  will need.
     *
     *  We can also start some sigs in here if we wanted to.
     *
     */
	public void init(){

		//This app requires special soft sensors/services.
		//so make them here.  This how to create our own soft
		//sensor service dynamically.
		Sensor sensor = getContextManager().getSensorManager().getSensor("SOUND");
		CallableSensor oktocall = new CallableSensor("OKTOCALL");
		oktocall.setSensorType( SensorOntology.OKTOCALL );
		sensor.addSensorListener(oktocall);
		getContextManager().addSensor(oktocall);


		//Sensor sensor2 = getContextManager().getSensorManager().getSensor("LIGHT");
		//CallableSensor oktotext = new CallableSensor("OKTOTEXT");
		//oktotext.setSensorType( 23 );
		//sensor2.addSensorListener(oktotext);
		//getContextManager().addSensor(oktotext);


	} //method: appSetUp




    /* ===================================================================== */
    /** This method is required by ContextApp.  This is where we are supposed
     *  to process sensor data that comes from local and remote sensors.
     *
     *  For this app we only care about the remote OKTOCALL sensor, so that
     *  we can know if it is okay to call some other agent.
     *
     */
	public void run(){
		try{Thread.sleep(1500);}catch(Exception e){}

		while(true){
			try{Thread.sleep(200);}catch(Exception e){}


			if(!getSensorQueue().isEmpty()){
				Object event = getSensorQueue().remove(0);
				SensorEvent se = (SensorEvent) event;
				if (se instanceof SensorDataEvent){
					SensorDataEvent dataE = (SensorDataEvent) se;

					String id = (String)(dataE.getSource());
					int type = dataE.getType();
					Value value = dataE.getValue();

					//check who the message is from, and record the value.
					//System.out.println("Receiving Sensor data from: "+id );

					if( type == SensorOntology.GPS ){


						Position chart_pos = new Position();
						Agent agent = new Agent(iSIM.ID, iSIM.PORT, new Profile(iSIM.PSUEDONAME,iSIM.LOCATION,iSIM.AVAILABLE) );
						if( dataE.isRemote() ){
							agent = dataE.getAgent();
						}else{
							iSIM.LOCATION = "UO campus";
						}

						double lat = ((int)(((Double)(value.getValue())).doubleValue()/100))*60;
						lat += (((Double)(value.getValue())).doubleValue() - (((int)(((Double)(value.getValue())).doubleValue()/100))*100));

						double lng = ((int)(((Double)(value.getSecondaryValue())).doubleValue()/100))*60;
						lng += (((Double)(value.getSecondaryValue())).doubleValue() - (((int)(((Double)(value.getSecondaryValue())).doubleValue()/100))*100));
						lng = lng * -1;


						chart_pos.setLatitude( lat );
						chart_pos.setLongitude( lng );

						table1.put(agent.getId(), chart_pos );
						try{
							table2.remove(agent.getId());
						}catch(Exception exc){}
						table2.put(agent.getId(), agent );

						//Point p = chartdata.transPos(chart_pos);
						//monitorUI.updatePos(p,chartdata.transPos(calcPred(chart_pos)));
						gui.updatePos(table1, iSIM.ID);
						gui.updateInfo(table2, iSIM.ID);



					}else if( type == SensorOntology.OKTOCALL ){

						Agent agent = dataE.getAgent();
						if( value.getValue() instanceof Boolean ){
							//pass the message to frame
							iSIM.AVAILABLE = ((Boolean)(value.getValue())).booleanValue();
						}

					}else if( type == SensorOntology.P2PDATA ){

						Agent agent = dataE.getAgent();
						if( value.getValue() instanceof String ){
							//pass the message to frame
							gui.incoming( agent, (String)(value.getValue()) );
						}

					}else  if( type == SensorOntology.DISCOVERY ){
						System.out.println("PhoneApp: Getting current list of agents near us:");
						Agent[] agents = (Agent[])(value.getValue());
						if( agents.length == 0 ){
							System.out.println("   empty");
						}else{
							for( int i=0; i<agents.length; i++){
								System.out.println("   "+agents[i].getId() +" "+ agents[i].getPort() );
								table2.put(agents[i].getId(), agents[i] );
							}
						}

						//Remove agents that are no longer in contact.
						java.util.Collection all = table2.keySet();
						Object[] objs = all.toArray();

						String aid ="";
						boolean check = false;
						for (int j=0; j<objs.length; j++) {
							aid = (String)objs[j];
							for( int i=0; i<agents.length; i++){
								if( aid.equals(agents[i].getId()) )
									check = true;
							}
							if( !check ){
								table2.remove(aid);
								table1.remove(aid);
							}
							check = false;
						}

					}else  if( type == SensorOntology.ENCOUNTER ){
						System.out.println("PhoneApp: Getting an encounter event:");
						Agent agent = (Agent)(value.getValue());

					  /*if( getContextManager().getSigManager().findSig("activecampus") != null ){
						  Vector want = new Vector();
						  want.add( new Integer(SensorOntology.GPS) );
						  getContextManager().getSigManager().addAgentToSig("activecampus", agent, want);
					  }else{
						  formSig(agent);
					  }*/
						System.out.println("   encountered agent " +agent);

					}

				}
		  	}
		} //while
	}

	/*public void formSig(Agent a){

		if( iSIM.PORT == 6000 ){

			System.out.println("   Creating a SIG with " +a);

			//pick a sensor that this app will share.
			Sensor toshare = null;

			//Need to find the OKTOCALL sensor that we'll share
			Sensor[] sensors = getContextManager().getSensorManager().getAllSensors();
			for(int i=0; i<sensors.length; i++ ){

				//want to share just this sensor
				if( sensors[i].getSensorType() == SensorOntology.GPS )
					toshare = sensors[i];
			}


			//create a sig group and share one sensor.
			SigManager sigman = getContextManager().getSigManager();

			Vector sharing = new Vector();
			sharing.add( toshare );
			Vector want = new Vector();
			want.add( new Integer(SensorOntology.GPS) );

			//System.out.println("Alice is creating a SIG with Bob in it.");
			//System.out.println("   Sending a SigRequest.");
			sigman.createSig("activecampus", a, want, sharing );
		}

	}*/


	public synchronized void formFriendSig(String id){

		Agent a = (Agent)(table2.get(id));

		System.out.println("   Creating a SIG with " +a);


		//Need to find the sensors that we'll share
		Vector sharing = new Vector();
		Sensor[] sensors = getContextManager().getSensorManager().getAllSensors();
		for(int i=0; i<sensors.length; i++ ){

			//want to share just this sensor
			if( sensors[i].getSensorType() == SensorOntology.OKTOCALL || sensors[i].getSensorType() == SensorOntology.GPS && !(sensors[i].isRemote()) )
				sharing.add( sensors[i] );
		}

		//pick the sensors that we want
		Vector want = new Vector();
		want.add( new Integer(SensorOntology.OKTOCALL) );
		want.add( new Integer(SensorOntology.GPS) );


		//create a sig group and share one sensor.
		SigManager sigman = getContextManager().getSigManager();


		//System.out.println("   Sending a SigRequest.");
		sigman.createSig("friend", a, want, sharing );

	}



	public void joinedSig(String name, Agent agent){
		if( name.equals("friend") ){
			gui.newFriend(agent.getId());
		}
	}



    /* ===================================================================== */
    /** We are asked by this method if this application would like to join
     *  a sig.
     *
     *  @param  name   The name of the SIG.
     *  @param  agent   The agent who wants us to join the SIG.
     *  @param
     *  @param
     *
     */
	public boolean joinSig(String name, Vector agents, Vector wanted_sensors, Vector sharing_sensors){

		this.name = name;
		this.agents = agents;
		this.wanted_sensors = wanted_sensors;
		this.sharing_sensors = sharing_sensors;

		//return true if we want to join the sig.
		//false otherwise.
		if( name.equals("friend") ){
			try{
			notification = new Dialog(gui, "Friend request");
			notification.setSize(200,100);
			Point pos = gui.getLocation();
			notification.setLocation(pos.x + 80, pos.y+200);

			Panel p = new Panel(null);
			p.setBounds( 0, 23, 200, 87);

			Label l = new Label("Be friends with "+((Agent)(agents.elementAt(0))).getId()+"?");
			l.setBounds( 10, 10, 180, 20);

			Button b = new Button("Yes");
			b.setBounds( 25, 35, 60, 23);

			Button b2 = new Button("No");
			b2.setBounds( 115, 35, 60, 23);

			p.setBackground(Color.white);
			p.add(l);
			p.add(b);
			p.add(b2);

			b.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(ActionEvent e) {
					notification.dispose();
					bic();
				}
			});
			b2.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(ActionEvent e) {
					notification.dispose();
				}
			});
			notification.addWindowListener(new WindowAdapter(){
				public void windowClosing(WindowEvent e){
					notification.dispose();
				}
			});

			notification.add(p);
			notification.setVisible(true);
			}catch(Exception e){}
			return false;


		}
		return true;
	}



	public void bic(){
		gui.newFriend( ((Agent)(agents.elementAt(0))).getId() );
		willJoinSig(name, agents, wanted_sensors, sharing_sensors);
	}



	public static void main(String[] args){

		String id = CommandLine.getArg(args, "a", "bob");
		String port = CommandLine.getArg(args, "p", "6001");

		//run PhoneApp with bob.
		new ActiveCampus(id, Integer.parseInt(port));

	} //method: main



} //class: GpsMonitor